/****************************************************************************\
*									     *
*			   Random number generators			     *
*									     *
*									     *
*			Marcel Leutenegger  1.11.2012			     *
*									     *
\****************************************************************************/

#include "random.h"


static	mxArray *mn, *mu, *sz;
static	const double *rn, *ru;
static	int nn, nu;


/*	Get an exponentially distributed random number: log(rand).
*/
double rande(void)
{	return log(randu());
}


/*	Get a normally distributed random number: randn.
*/
double randn(void)
{	if (--nn < 0)
	{	if (mn) mxDestroyArray(mn);
		mexCallMATLAB(1,&mn,1,&sz,"randn");
		mexMakeArrayPersistent(mn);
		rn=mxGetPr(mn);
		nn=8191;
	}
	return rn[nn];
}


/*	Approximately Poisson distributed random number.
*/
double randp(double lambda)
{	if (isnan(lambda) || lambda < 0) return NaN;
	if (lambda > 36) return fix(lambda + sqrt(lambda)*randn());
	int rnd=0;
	while((lambda+=rande()) > 0) rnd+=1;
	return rnd;
}


/*	Get a uniformly distributed random number: rand.
*/
double randu(void)
{	if (--nu < 0)
	{	if (mu) mxDestroyArray(mu);
		mexCallMATLAB(1,&mu,1,&sz,"rand");
		mexMakeArrayPersistent(mu);
		ru=mxGetPr(mu);
		nu=8191;
	}
	return ru[nu];
}


/*	Initialize random number generators.
*/
void init(void)
{	if (sz == 0)
	{	mu=mn=0;
		rn=ru=0;
		nn=nu=0;
		mexAtExit(done);
		double* s=mxGetPr(sz=mxCreateDoubleMatrix(1,2,mxREAL));
		mexMakeArrayPersistent(sz);
		s[0]=8192;
		s[1]=1;
	}
}


/*	Destroy random number generators.
*/
void done(void)
{	if (sz) mxDestroyArray(sz);
	if (mn) mxDestroyArray(mn);
	if (mu) mxDestroyArray(mu);
	sz=mn=mu=0;
}
