/****************************************************************************\
*									     *
*			   Random number generators			     *
*									     *
*									     *
*			Marcel Leutenegger  1.11.2012			     *
*									     *
\****************************************************************************/

#include "random.h"


static	mxArray *mn, *mu, *in[2];
static	const float *rn, *ru;
static	int nn, nu;


/*	Get an exponentially distributed random number: log(rand).
*/
float rande(void)
{	return logf(randu());
}


/*	Get a normally distributed random number: randn.
*/
float randn(void)
{	if (--nn < 0)
	{	if (mn) mxDestroyArray(mn);
		mexCallMATLAB(1,&mn,2,in,"randn");
		mexMakeArrayPersistent(mn);
		rn=mxGetData(mn);
		nn=65535;
	}
	return rn[nn];
}


/*	Approximately Poisson distributed random number.
*/
float randp(float lambda)
{	if (isnanf(lambda) || lambda < 0) return NAN;
	if (lambda > 36) return floorf(lambda + sqrtf(lambda)*randn());
	int rnd=0;
	while((lambda+=rande()) > 0) rnd+=1;
	return rnd;
}


/*	Get a uniformly distributed random number: rand.
*/
float randu(void)
{	if (--nu < 0)
	{	if (mu) mxDestroyArray(mu);
		mexCallMATLAB(1,&mu,2,in,"rand");
		mexMakeArrayPersistent(mu);
		ru=mxGetData(mu);
		nu=65535;
	}
	return ru[nu];
}


/*	Initialize random number generators.
*/
void init(void)
{	if (in[0] == 0)
	{	mu=mn=0;
		rn=ru=0;
		nn=nu=0;
		mexAtExit(done);
		mexMakeArrayPersistent(in[0]=mxCreateDoubleMatrix(1,2,mxREAL));
		mexMakeArrayPersistent(in[1]=mxCreateString("single"));
		double* s=mxGetData(in[0]);
		s[0]=65536;
		s[1]=1;
	}
}


/*	Destroy random number generators.
*/
void done(void)
{	if (in[0]) mxDestroyArray(in[0]);
	if (in[1]) mxDestroyArray(in[1]);
	if (mn) mxDestroyArray(mn);
	if (mu) mxDestroyArray(mu);
	in[0]=in[1]=mn=mu=0;
}
