%% Brownian motion in three dimensions:
%
% P(r,r',tau) = <C>(4pi*D*tau)^(-3/2) exp[-|r-r'|^2/(4D*tau)]
%
% <dr> = sqrt(6D*tau)
% <dx> = sqrt(2D*tau)
%

%% Differential equations with flow:
%
% <ds> = sqrt(2D*dt)
% dx/dt = <ds>*randn
% dy/dt = <ds>*randn
% dz/dt = vz(x,y) + <ds>*randn
%

%% Monte-Carlo simulation of trajectories
%
t=0;
f=statusbar('Trajectories...');
for m=1:100
   r=cumsum(complex(randn(1e7,1),randn(1e7,1)));
   for n=1:10:1e4
      t=t + histc(abs(r(n:n+999)-r(n)),0:150);
   end
   clear('r','n');
   f=statusbar(m/100,f);
   figure(1);
   plot(0:150,t/(1e6*m),'r.');
   if isempty(f)
      break;
   end
end
delete(f);
t=t(1:end-1)/(1e6*m);
r=(0.5:numel(t)).';
clear('f','m');
