%[P,z]=flowDistance(x,y,z,v,D,t)
%-------------------------------
%
%Probability P of flowing a distance z.
%
%Inputs:
% x,y,z  Sampling points [m]
% v(x,y) Flow speed along z
% D      Diffusion coefficient [m/s]
% t      Lag times [s]
%
%Outputs:
% P      Probabilities
% z      Flow distances [m]
%
function [P,z]=flowDistance(x,y,z,v,D,t)
A=mean(diff(x)).*mean(diff(y));

t=t(:).';
[x,y]=ndgrid(x.*x,y.*y);
d=4*pi*D*t;
d=A./(sqrt(d).*d);
p=exp((x(:)+y(:))*(-0.25./(D*t)));     % Brownian motion
p=p.*repmat(d,numel(x),1);
p=p.*repmat(min(1,1./sum(p,1)),numel(x),1);

d=v(:)*[t(1) t(2:end)-t(1:end-1)];     % Incremental flow distances

[~,m]=histc(d,z,1);
T=numel(t);
Z=numel(z);
P=zeros(Z,T);
for n=1:T
   P(:,n)=accumarray(m(:,n),p(:,n),[Z 1]);
end

for n=2:T
   p=conv(P(:,n-1),P(:,n));
   P(:,n)=p(1:Z);
end
