%I=occsDetect(I,scale,noise,bias,bits,ref)
%-----------------------------------------
%
%Detected intensity: convert intensity to digital values.
%
%Inputs:
% I      Intensity [photons]
% scale  Conversion scale [units/photon]        {0.1}
% noise  Read noise standard deviation [units]  {1}
% bias   Bias/offset value [units]              {2*noise}
% bits   Number of bits in converted values     {12}
% ref    Reference spectrum if set (average)
%
%Output:
% I      Detected values
%
function I=occsDetect(I,scale,noise,bias,bits,ref)
if nargin < 2 || isempty(scale)
   scale=0.1;
end
if nargin < 3 || isempty(noise)
   noise=1;
end
if nargin < 4 || isempty(bias)
   bias=2*abs(noise);
end
if nargin < 5 || isempty(bits)
   bits=12;
end
if nargin == 6 && ref
   I=bias + scale*I;
   I=(noise/sqrt(2*pi)*exp(-0.5/noise^2*I.^2) + 0.5*I.*(1 + erf(I/(noise*sqrt(2)))))./(1 - normcdf(0,I,noise));
else
   try
      cls=classUnderlying(I);
   catch
      cls=class(I);
   end
   I=scale*poissrand(I,cls);
   if bias
      I=I + bias;
   end
   if noise
      I=I + noise*randn(size(I),cls);
   end
   I=max(0,min(round(I),pow2(1,bits)-1));
end


%Random numbers from Poisson distribution for lambda < 36
%and from the approximating normal distribution otherwise.
%
function r=poissrand(lambda,cls)
r=randn(size(lambda),cls);
r=fix(lambda + sqrt(lambda).*r);
k=lambda < 36;
r(k)=randp(gather(lambda(k)));
