%[A,z]=occsTomogram(Id,kn,zs)
%----------------------------
%
%Tomogram amplitude |A(z)| from interference spectrum Id(kn).
%
%Inputs:
% Id     Detected interference spectrum
% kn     Wavevector in medium [rad/m] (strictly increasing)
% zs     Axial sample indices to retain
%
%Outputs:
% A      Tomogram amplitude
% z      Sample positions [m]
%
function [A,z]=occsTomogram(Id,kn,zs)
persistent r s u v
if nargin > 1 && ~isempty(kn)
   N=numel(kn);
   v=kn(1) + (kn(end)-kn(1))/(2*N-1)*(0:2*N-1).';
   [~,r]=histc(gather(v),gather(kn));
   r=min(r,N-1);
   s=int32(r+1);
   r=int32(r);
   u=kn(r);
   u=(v-u)./(kn(s)-u);
   v=1 - u;
end
A=ones(1,size(Id,2),'int32');
A=Id(r,:).*v(:,A) + Id(s,:).*u(:,A);
A=fft(A,[],1);
% N=numel(kn);
% k=kn(1) + (kn(end)-kn(1))/(2*N-1)*(0:2*N-1);
% A=fft(interp1q(kn,Id,k(:)),2*N,1);
if nargin < 3 || isempty(zs)
   zs=1:N;
else
   A=A(zs,:);
end
A=abs(A);
if nargout > 1
   z=pi/abs(kn(end)-kn(1))*(zs(:)-1);
end


%Quick 1D linear interpolation.
%
% x,xi   Strictly increasing
%
% function y=interp1q(x,y,xi)
% n=numel(x);
% [~,r]=histc(xi,x);
% r=min(r,n-1);
% u=x(r);
% u=(xi-u)./(x(r+1)-u);
% v=1 - u;
% n=ones(1,size(y,2));
% y=y(r,:).*v(:,n) + y(r+1,:).*u(:,n);
