/*	>> !nvcc -arch=sm_21 -ptx interferogram.cu

	The file interferogram.ptx may require a change: each global function is defined
	by ".entry" followed by the mangled function name. MATLAB 2012a requires a space
	between the function name and the argument bracket.
*/


/*	zs=reshape(zs,1,numel(zs));
	Es=reshape(Es,size(zs));
	Es=(Er*Es).*exp((2i*kn)*(zs-zr));
	Es=sum(reshape(Es,R,T,S),3);
	Ic=abs(Er(:,ones(1,T)) + Es).^2;

		Er	Reference spectrum [M x 1]
		Es	Sample fields [N x S]
		kr	Wavenumbers 2/pi*kn [M x 1]
		zs	Sample positions [N x S]
		zr	Reference position [scalar]
		task	Dimensions and reference [M N S zr]

		Ic	Interferogram [M x N]
*/
__global__ void interferogram(double* Ic, const double* Er, const double* kr, const double* Es, const double* zs, const double* task)
{	int m=threadIdx.x + blockDim.x*blockIdx.x;
	int M=(int)task[0];
	int n=threadIdx.y + blockDim.y*blockIdx.y;
	int N=(int)task[1];
	if(m < M && n < N)
	{	int k=m + M*n;
		int S=(int)task[2];
		double kk=kr[m], zr=task[3], Re, Im;
		sincospi(kk*(zs[n] - zr),&Im,&Re);
		Re=Re*Es[n] + 1;
		Im*=Es[n];
		while(--S > 0)
		{	double re, im;
			sincospi(kk*(zs[n+=N] - zr),&im,&re);
			Re+=re*Es[n];
			Im+=im*Es[n];
		}
		Ic[k]=(Re*Re + Im*Im)*Er[m]*Er[m];
	}
}


__global__ void interferogramf(float* Ic, const float* Er, const float* kr, const float* Es, const float* zs, const double* task)
{	int m=threadIdx.x + blockDim.x*blockIdx.x;
	int M=(int)task[0];
	int n=threadIdx.y + blockDim.y*blockIdx.y;
	int N=(int)task[1];
	if(m < M && n < N)
	{	int k=m + M*n;
		int S=(int)task[2];
		float kk=kr[m], zr=(float)task[3], Re, Im;
		sincospif(kk*(zs[n] - zr),&Im,&Re);
		Re=Re*Es[n] + 1;
		Im*=Es[n];
		while(--S > 0)
		{	float re, im;
			sincospif(kk*(zs[n+=N] - zr),&im,&re);
			Re+=re*Es[n];
			Im+=im*Es[n];
		}
		Ic[k]=(Re*Re + Im*Im)*Er[m]*Er[m];
	}
}
